/*
 * File: envars.c
 * Date: 2011-11-01
 *
 * Evsiz (evsiz@this-voice.org)
 * Rho released under the Creative Commons NonCommercial ShareAlike License 3.0
 * See http://creativecommons.org for details
 *
 * Rho is a fork of the Linux port of MicroEMACS developed by Humberto
 * Ortiz-Zuazaga and Jean-Pierre Demailly. Their work forked the MicroEMACS
 * of Daniel M. Lawrence which was in turn a fork of uemacs by Dave G.
 * Conroy. MicroEmacs was copyrighted 1993 by Mr. Lawrence with a
 * stipulation that it could not be used commercially. Uemacs was released
 * by Mr. Conroy into the public domain in 1985.
 *
 */

/***

The original MicroEMACS code exploits string arrays and #defines
to create primitive dictionary structures.  This file uses envars[]
(initialize in rho_init.h and enumerated by defines in rho.h) in this way.

***/


#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	"rho.h"
#include	"envars.h"
#include	"bind.h"
#include	"buffer.h"
#include	"exec.h"
#include	"line.h"
#include	"macros.h"
#include	"messages.h"
#include	"terminal.h"
#include	"utils.h"
#include	"window.h"



/*++

Place in a buffer list of characters considered "in a word"

++*/
static char *getwlist(buf)
char *buf;                      /* buffer to place list of characters */

{
    int index;
    char *sp;

    /* if defaulting to standard word list */
    if (wlflag == FALSE)
        return ("");

    /* build the string of characters in the return buffer */
    sp = buf;
    for (index = 0; index < 256; index++)
        if (wordlist[index])
            *sp++ = index;
    *sp = 0;
    return (buf);
}



/*++

Evaluate envar bound to string "vname" in envars[] in rho_init.h

++*/
char *eval_envar(vname)
char *vname;                    /* name of environment variable to retrieve */

{
    int vnum;                   /* ordinal number of var refrenced */
    static char result[2 * NSTRING];    /* string result */

    /* scan the list, looking for the referenced name */
    vnum = index_of_str(vname, get_envars_name, nenvars);

    /* return errorm on a bad reference */
    if (vnum == -1)
        return ((char *) errorm);

    /* otherwise, fetch the appropriate value */
    switch (vnum) {
    case EVASAVE:
        return (int2str(asave));
    case EVBANGLINE:
        return (bangline);
    case EVBUFHOOK:
        return (fixnull(get_name(&bufhook)));
    case EVCFNAME:
        return (curbp->b_name);
    case EVCMDHK:
        return (fixnull(get_name(&cmdhook)));
    case EVCOMPILER:
        return (compiler);
    case EVDEBUG:
        return (bool2str(debug));
    case EVEXBHOOK:
        return (fixnull(get_name(&exbhook)));
    case EVFILLCOL:
        return (int2str(fillcol));
    case EVFMTOPTS:
        return (fmtopts);
    case EVHARDTAB:
        return (int2str(hardtab));
    case EVREADHK:
        return (fixnull(get_name(&readhook)));
    case EVREPLACE:
        return ((char *) rpat);
    case EVRVAL:
        return (int2str(rval));
    case EVSEARCH:
        return ((char *) pat);
    case EVSOFTTAB:
        return (int2str(softtab));
    case EVSORTOPTS:
        return (sortopts);
    case EVSTATUS:
        return (bool2str(cmdstatus));
    case EVTAGCMD:
        return (tagcmd);
    case EVTIME:
        return (getctime());
    case EVWCHARS:
        return (getwlist(result));
    case EVWRAPHK:
        return (fixnull(get_name(&wraphook)));
    case EVWRITEHK:
        return (fixnull(get_name(&writehook)));
    }
    return ("");                /* shouldn't get here */
}



/*++

Return name string of envars[i] rho_init.h

++*/
char *get_envars_name(i)
int i;

{
    return (envars[i]);
}



/*++

Set envar in envars[] in rho_init.h
Var to change comes from macro or interactively from the message-line.

++*/
int set_envar(f, n)
int f;                          /* default flag */
int n;                          /* numeric arg (can overide prompted value) */

{
    int status;                 /* status return */
    VDESC vd;                   /* variable num/type */
    char var[NVSIZE + 1];       /* name of variable to fetch */
    char value[NSTRING];        /* value to set variable to */

    /* first get the variable to set.. */
    if (clexec == FALSE) {
        status = mlreply(TEXT51, &var[0], NVSIZE + 1);  /* "Variable to set: " */
        if (status != TRUE)
            return (status);
    } else {                    /* macro line argument */
        /* grab token and skip it */
        execstr = next_token(execstr, var, NVSIZE + 1);
    }

    /* check the legality and find the var */
    find_var(var, &vd, NVSIZE + 1);

    /* if its not legal....bitch */
    if (vd.v_type == -1) {
        mlwrite(TEXT52, var);   /* "%%No such variable as '%s'" */

        return (FALSE);
    }

    /* get the value for that variable */
    if (f == TRUE)
        strcpy(value, int2str(n));
    else {
        status = mlreply(TEXT53, &value[0], NSTRING);   /* "Value: " */
        if (status == ABORT)
            return (status);
    }

    /* and set the appropriate value */
    status = set_var(&vd, value);

    /* and return it */
    return (status);
}



/*++

Set an alternative list of characters to be considered "in a word."

++*/
void setwlist(wclist)
char *wclist;                   /* list of characters considered "in a word" */

{
    int index;

    /* if turning this off, set flag */
    if (wclist == NULL || *wclist == 0) {
        wlflag = FALSE;
        return;
    }

    /* first clear the table */
    for (index = 0; index < 256; index++)
        wordlist[index] = FALSE;

    /* and for each character in the new value, set that element
       of the word character list */
    while (*wclist)
        wordlist[(unsigned char) (*wclist++)] = TRUE;   /* ep */
    wlflag = TRUE;
    return;
}


/*++

Pad a string to indicated length

++*/
static void pad(s, len)
char *s;                        /* string to add spaces to */
int len;                        /* wanted length of string */

{
    while ((int) strlen(s) < len) {
        strcat(s, "          ");
        s[len] = 0;
    }
}



/*++

List envars and their values in a popup

++*/
int describe_envars(f, n)
int f, n;

{
    BUFFER *varbuf;             /* buffer to put variable list into */
    int uindex;                 /* index into uvar table */
    char outseq[256];           /* output buffer for keystroke sequence */

    f = n = 0;                  /* splint */

    /* and get a buffer for it */
    varbuf = exist_buffer(TEXT56, TRUE, BFINVS);        /* "Variable list" */
    if (varbuf == NULL || clear_buffer(varbuf) == FALSE) {
        mlwrite(TEXT57);        /* "Can not display variable list" */
        return (FALSE);
    }


    /* build the environment variable list */
    for (uindex = 0; uindex < nenvars; uindex++) {

        /* add in the environment variable name */
        strcpy(outseq, "$");
        strcat(outseq, envars[uindex]);
        pad(outseq, 12);

        /* add in the value */
        strcat(outseq, eval_envar(envars[uindex]));

        /* and add it as a line into the buffer */
        if (addline(varbuf, outseq) != TRUE)
            return (FALSE);
    }

    if (addline(varbuf, "") != TRUE)
        return (FALSE);

    /* build the user variable list */
    for (uindex = 0; uindex < MAXUVARS; uindex++) {
        if (uv[uindex].u_name[0] == 0)
            break;

        /* add in the user variable name */
        strcpy(outseq, "%");
        strcat(outseq, uv[uindex].u_name);
        pad(outseq, 12);

        /* add in the value */
        strcat(outseq, uv[uindex].u_value);

        /* and add it as a line into the buffer */
        if (addline(varbuf, outseq) != TRUE)
            return (FALSE);
    }

    /* display the list */
    wpopup(varbuf);
    mlerase();                  /* clear the mode line */
    return (TRUE);
}
